@@ -59,7 +59,7 @@ pnormAsymp <- function(x, k, lower.tail=FALSE, log.p=FALSE) {
     ##     x <- -x
     ##     lower.tail <- ! lower.tail
     ## }
-    r <- dnorm(x, log=TRUE) - log(x)
+    r <- dnorm(x, log=TRUE) - log(x) ## <==> exp(r) = phi(x)/x
     if(k > 0) {
         xsq <- x*x
         del <-
@@ -92,6 +92,10 @@ pnormAsymp <- function(x, k, lower.tail=FALSE, log.p=FALSE) {
 
 
 
+###----------- NB>  qnormAppr(), qnormUappr() and qnormUappr6() -->>>> ./beta-fns.R <<<<<<
+##                  =========    ==========       ===========            ~~~~~~~~~~
+
+
 ### R version of   ~/R/D/r-devel/R/src/nmath/qnorm.c
 
 ## Mathlib : A C Library of Special Functions
@@ -141,14 +145,16 @@ pnormAsymp <- function(x, k, lower.tail=FALSE, log.p=FALSE) {
 ##     Applied Statistics, 37, 477-484.
 
 
+## <--> ../man/qnormR.Rd
+
 qnormR1 <- function(p, mu=0, sd=1, lower.tail=TRUE, log.p=FALSE,
-                    trace = 0, version = c("4.0.x", "2020-10-17")) {
+                    trace = 0, version = c("4.0.x", "2020-10-17", "2022-08-04")) {
     stopifnot(length(p) == 1, length(mu) == 1, length(sd) == 1,
               length(lower.tail) == 1, length(log.p) == 1,
               !is.na(lower.tail), !is.na(log.p))
     version <- match.arg(version)
     if(is.na(p) || is.na(mu) || is.na(sd)) return(p+mu+sd)
-    ## R_Q_P01_boundaries(p, ML_NEGINF, ML_POSINF);
+    ## R_Q_P01_boundaries(p, ML_NEGINF, ML_POSINF) :
     if(p == .D_0(log.p)) return(if(lower.tail) -Inf else  Inf)
     if(p == .D_1(log.p)) return(if(lower.tail)  Inf else -Inf)
     if(p < .D_0(log.p) ||
@@ -175,7 +181,8 @@ qnormR1 <- function(p, mu=0, sd=1, lower.tail=TRUE, log.p=FALSE,
 ##        and provided hash codes for checking them...)
 
     if (abs(q) <= .425) { ## 0.075 <= p <= 0.925 */
-        r  <- .180625 - q * q;
+        r  <- .180625 - q * q; ## History: AS241 has SPLIT1 = 0.425, CONST1 = 0.180625
+        if(trace) cat(sprintf(" --> usual rational form 1, r=%12.7g\n", r))
 	val <-
             q * (((((((r * 2509.0809287301226727 +
                        33430.575583588128105) * r + 67265.770927008700853) * r +
@@ -189,8 +196,8 @@ qnormR1 <- function(p, mu=0, sd=1, lower.tail=TRUE, log.p=FALSE,
                  687.1870074920579083) * r + 42.313330701600911252) * r + 1.)
     }
     else { ## closer than 0.075 from {0,1} boundary */
-           ##   r := log(p~);  p~ = min(p, 1-p) < 0.075 :
-	r <-
+           ## "r" := lp := log(p~);  p~ = min(p, 1-p) < 0.075 :
+	lp <-
             if(log.p && ((lower.tail && q <= 0) || (!lower.tail && q > 0))) {
                 p
             } else {
@@ -198,14 +205,14 @@ qnormR1 <- function(p, mu=0, sd=1, lower.tail=TRUE, log.p=FALSE,
                      else p. ) # p. = R_DT_Iv(p) ^=  p
             }
 	###  r = sqrt( - log(min(p,1-p)) )  <==>  min(p, 1-p) = exp( - r^2 ) :
-        r  <- sqrt(-r)
-        if(trace)
-            cat(sprintf("\t close to 0 or 1: r = %7g\n", r))
+        r  <- sqrt(-lp)
+        if(trace) cat(sprintf("\t somewhat close to 0 or 1: r := sqrt(-lp) = %11g\n", r))
         if(is.na(r))
             warning("r = sqrt( - log(min(p,1-p)) )  is NA -- \"we have lost\"")
 
         if (!is.na(r) && r <= 5.) { ## <==> min(p,1-p) >= exp(-25) ~= 1.3888e-11 */
-            r <- r + -1.6;
+            r <- r + -1.6; ## History: AS241 has SPLIT2 = 5.0, CONST2 = 1.6
+            if(trace) cat(sprintf("\t as r <= 5, using rational form 2, for (updated) r=%12.7g\n", r))
             val <- (((((((r * 7.7454501427834140764e-4 +
                        .0227238449892691845833) * r + .24178072517745061177) *
                      r + 1.27045825245236838258) * r +
@@ -224,8 +231,43 @@ qnormR1 <- function(p, mu=0, sd=1, lower.tail=TRUE, log.p=FALSE,
              ## Using the asymptotical formula -- is *not* optimal but uniformly better than branch below
              val = r * M_SQRT2;
           }
-          else {
+          else if(version == "2022-08-04" && r > 27) {
+              ## p is *really* close to 0 or 1 .. practically only when log_p =TRUE
+            if(trace) cat(sprintf("\t *really* close to 0 or 1; ==> using an asymptotic formula; "))
+	    if(r >= 6.4e8) { ## p is *very extremly* close to 0 or 1
+		## Using the asymptotical formula ("0-th order"): qn = sqrt(2*s)
+                if(trace) cat(sprintf("  r large \n"))
+		val = r * M_SQRT2;
+	    } else {
+		s2 <- -ldexp(lp, 1) ## = -2*lp = 2s
+                x2 <- s2 - log(M_2PI * s2); ## = xs_1
+                if(trace) cat(sprintf("  1st order x2=%11g \n", x2))
+		## if(r >= 36000.)  # use x2 = xs_1 above
+		if(r < 36000.) { ## <==> s < 36000^2
+		    x2 = s2 - log(M_2PI * x2) - 2./(2. + x2); ## == xs_2
+                    if(trace) cat(sprintf("  2nd order x2=%11g \n", x2))
+		    if(r < 840.) { ## 27 < r < 840
+			x2 = s2 - log(M_2PI * x2) + 2*log1p(- (1 - 1/(4 + x2))/(2. + x2)); ## == xs_3
+                        if(trace) cat(sprintf("  3rd order x2=%11g \n", x2))
+			if(r < 109.) { ## 27 < r < 109
+			  x2 = s2 - log(M_2PI * x2) +
+			      2*log1p(- (1 - (1 - 5/(6 + x2))/(4. + x2))/(2. + x2)); ## == xs_4
+                          if(trace) cat(sprintf("  4-th order x2=%11g \n", x2))
+			  if(r < 55.) { ## 27 < r < 55
+                            if(trace) cat("27 < r < 55: using 5-th order x2\n")
+			    x2 = s2 - log(M_2PI * x2) +
+			      2*log1p(- (1 - (1 - (5 - 9/(8. + x2))/(6. + x2))/(4. + x2))/(2. + x2)); ## == xs_5
+			  }
+			}
+		    }
+		}
+                val = sqrt(x2);
+	    }
+          }
+          else { ## r > 5 and depending on version  r < **
+	    ## Wichura, p.478: minimax rational approx R_3(t) is for 5 <= t <= 27  (t :== r)
             r <- r + -5.;
+            if(trace) cat(sprintf("\t r > 5, using rational form R_3(t), for t=%12.7g\n", r))
             val = (((((((r * 2.01033439929228813265e-7 +
                        2.71155556874348757815e-5) * r +
                       .0012426609473880784386) * r + .026532189526576123093) *
@@ -241,11 +283,59 @@ qnormR1 <- function(p, mu=0, sd=1, lower.tail=TRUE, log.p=FALSE,
           }
         }
 	if(q < 0)
-	    val  <- -val;
-        ## return (q >= 0.)? r : -r ;*/
+	    val <- -val;
     }
-    return(mu + sd * val)
+    ## return
+    mu + sd * val
 } # qnormR1()
 
-qnormR  <- Vectorize(qnormR1, c("p", "mu", "sd"))
+## FIXME:    Rmpfr / mpfrize  as bd0() in  --> ./dgamma.R
+## FIXME(2): In that case, need N(.) for sprintf(.. %g ..)  -- or even just format(<num>)
+qnormR <- Vectorize(qnormR1, c("p", "mu", "sd"))
 ##==
+
+### Partly modeled after qnormUappr()  from ./beta-fns.R
+
+qnormAsymp <- function(p, lp = .DT_Clog(p, lower.tail=lower.tail, log.p=log.p),
+                           # ~= log(1-p) -- independent of lower.tail, log.p
+                       order,
+                       lower.tail = TRUE, log.p = missing(p))
+{
+    stopifnot(length(order) == 1L, order == (ord <- as.integer(order)), 0L <= ord, ord <= 5L)
+    if(missing(p)) { ## log.p unused;  lp = log(1-p)  <==>  e^lp = 1-p  <==>  p = 1 - e^lp
+        p. <- -expm1(lp)
+        ## swap p <--> 1-p -- so we are where approximation is better
+        swap <- if(lower.tail) p. < 1/2 else p. > 1/2 # logical vector
+    } else {
+        p. <- .D_qIv(p, log.p)
+        ## swap p <--> 1-p   (as above)
+        swap <- if(lower.tail) p. < 1/2 else p. > 1/2 # logical vector
+        p[swap] <- if(log.p) log1mexp(-p[swap]) else 1 - p[swap]
+    }
+    iFin <- is.finite(lp)
+    ##  r = sqrt( - log(min(p,1-p)) )  <==>  min(p, 1-p) = exp( - r^2 )
+    x2 <- s2 <- -ldexp(lp[iFin], 1) ## = -2*lp = 2s =: xs_0
+    if(ord >= 1L) {
+        x2 <- s2 - log(M_2PI * x2); ## = xs_1
+        if(ord >= 2L) { ## need for (r < 36000.)  <==> s < 36000^2
+            x2 = s2 - log(M_2PI * x2) - 2./(2. + x2); ## == xs_2
+            if(ord >= 3L) { ## need for (r < 840)
+                x2 = s2 - log(M_2PI * x2) + 2*log1p(- 1./(2. + x2)*(1 - 1/(4 + x2))); ## == xs_3
+                if(ord >= 4L) { ## need for (r < 109)
+                    x2 = s2 - log(M_2PI * x2) + 2*log1p(- 1./(2. + x2)*(1 - 1/(4. + x2)*(1 - 5/(6 + x2)))); ## == xs_4
+                    if(ord >= 5L) { ## need for (r < 55)
+                        x2 = s2 - log(M_2PI * x2) + 2*log1p( - 1./(2. + x2)*
+                                                            (1 - 1/(4. + x2)*
+                                                             (1 - 1/(6. + x2)*
+                                                              (5 - 9/(8. + x2))))); ## == xs_5
+                    }
+                }
+            }
+        }
+    }
+    R <- -lp # incl. Inf
+    R[iFin] <- sqrt(x2)
+    R[swap] <- -R[swap]
+    ## R[p. == 1/2] <- 0
+    R
+}
