@@ -18,7 +18,7 @@
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation; either version 3 of the License, or
  *  (at your option) any later version.
  *
  *  This program is distributed in the hope that it will be useful,
@@ -412,13 +412,15 @@ SEXP dpq_ebd0(SEXP x_, SEXP np_, SEXP trace_)
 	n_x  = XLENGTH(x_),
 	n_np  = XLENGTH(np_),
 	n = (n_x >= n_np ? n_x : n_np);
+    if(n > INT_MAX) // as R's matrices cannot yet have long dim()
+	error("length() of 'x' or 'np' = %ld > max.int = %d", n, INT_MAX);
     if(!n_x || !n_np) return allocVector(REALSXP, 0); // length 0
     if(length(trace_) != 1)
 	error("'length(%s)' must be 1, but is %d", "trace",  length(trace_));
     // otherwise, recycle (x, np) to common length n :
     PROTECT(x_  = isReal(x_)  ? x_  : coerceVector(x_,  REALSXP));
     PROTECT(np_ = isReal(np_) ? np_ : coerceVector(np_, REALSXP));
-    SEXP r_ = PROTECT(allocMatrix(REALSXP, 2, n)); // result =^= rbind(yh=yh, yl=yl)
+    SEXP r_ = PROTECT(allocMatrix(REALSXP, 2, (int)n)); // result =^= rbind(yh=yh, yl=yl)
     double *x = REAL(x_), *np = REAL(np_)
 	/* , delta = asReal(delta_) */
 	, *r = REAL(r_);
